@@ -16,11 +16,11 @@
 package org.gradle.api.internal.artifacts.ivyservice.moduleconverter;
 
 import org.gradle.api.artifacts.Configuration;
-import org.gradle.internal.component.local.model.MutableLocalComponentMetaData;
+import org.gradle.internal.component.local.model.BuildableLocalComponentMetaData;
 
 public class DefaultConfigurationsToArtifactsConverter implements ConfigurationsToArtifactsConverter {
 
-    public void addArtifacts(MutableLocalComponentMetaData metaData, Iterable<? extends Configuration> configurations) {
+    public void addArtifacts(BuildableLocalComponentMetaData metaData, Iterable<? extends Configuration> configurations) {
         for (Configuration configuration : configurations) {
             metaData.addArtifacts(configuration.getName(), configuration.getArtifacts());
         }
