@@ -16,8 +16,11 @@
 package org.gradle.launcher.daemon.configuration;
 
 import com.google.common.collect.ImmutableList;
+import org.gradle.api.JavaVersion;
+import org.gradle.api.Nullable;
 import org.gradle.api.internal.file.IdentityFileResolver;
 import org.gradle.initialization.BuildLayoutParameters;
+import org.gradle.internal.jvm.JavaInfo;
 import org.gradle.internal.jvm.Jvm;
 import org.gradle.process.internal.JvmOptions;
 import org.gradle.util.GUtil;
@@ -25,12 +28,11 @@ import org.gradle.util.GUtil;
 import java.io.File;
 import java.util.*;
 
-import static org.gradle.util.GFileUtils.canonicalise;
-
 public class DaemonParameters {
     static final int DEFAULT_IDLE_TIMEOUT = 3 * 60 * 60 * 1000;
 
     public static final List<String> DEFAULT_JVM_ARGS = ImmutableList.of("-Xmx1024m", "-XX:MaxPermSize=256m", "-XX:+HeapDumpOnOutOfMemoryError");
+    public static final List<String> DEFAULT_JVM_9_ARGS = ImmutableList.of("-Xmx1024m", "-XX:+HeapDumpOnOutOfMemoryError");
     public static final String INTERACTIVE_TOGGLE = "org.gradle.interactive";
 
     private final String uid;
@@ -40,10 +42,11 @@ public class DaemonParameters {
     private int idleTimeout = DEFAULT_IDLE_TIMEOUT;
     private final JvmOptions jvmOptions = new JvmOptions(new IdentityFileResolver());
     private DaemonUsage daemonUsage = DaemonUsage.IMPLICITLY_DISABLED;
-    private File javaHome;
+    private boolean hasJvmArgs;
     private boolean foreground;
     private boolean stop;
     private boolean interactive = System.console() != null || Boolean.getBoolean(INTERACTIVE_TOGGLE);
+    private JavaInfo jvm = Jvm.current();
 
     public DaemonParameters(BuildLayoutParameters layout) {
         this(layout, Collections.<String, String>emptyMap());
@@ -51,7 +54,6 @@ public class DaemonParameters {
 
     public DaemonParameters(BuildLayoutParameters layout, Map<String, String> extraSystemProperties) {
         this.uid = UUID.randomUUID().toString();
-        jvmOptions.setAllJvmArgs(DEFAULT_JVM_ARGS);
         jvmOptions.systemProperties(extraSystemProperties);
         baseDir = new File(layout.getGradleUserHomeDir(), "daemon");
         gradleUserHomeDir = layout.getGradleUserHomeDir();
@@ -90,32 +92,25 @@ public class DaemonParameters {
         return jvmOptions.getAllImmutableJvmArgs();
     }
 
-    public List<String> getAllJvmArgs() {
-        return jvmOptions.getAllJvmArgs();
+    public JavaInfo getEffectiveJvm() {
+        return jvm;
     }
 
-    public File getEffectiveJavaHome() {
-        if (javaHome == null) {
-            return canonicalise(Jvm.current().getJavaHome());
-        }
-        return javaHome;
-    }
-
-    public File getEffectiveJavaExecutable() {
-        if (javaHome == null) {
-            return Jvm.current().getJavaExecutable();
-        }
-        return Jvm.forHome(javaHome).getJavaExecutable();
-    }
-
-    public DaemonParameters setInteractive(boolean interactive) {
-        this.interactive = interactive;
+    @Nullable
+    public DaemonParameters setJvm(JavaInfo jvm) {
+        this.jvm = jvm == null ? Jvm.current() : jvm;
         return this;
     }
 
-    public DaemonParameters setJavaHome(File javaHome) {
-        this.javaHome = javaHome;
-        return this;
+    public void applyDefaultsFor(JavaVersion javaVersion) {
+        if (hasJvmArgs) {
+            return;
+        }
+        if (javaVersion.compareTo(JavaVersion.VERSION_1_9) >= 0) {
+            jvmOptions.setAllJvmArgs(DEFAULT_JVM_9_ARGS);
+        } else {
+            jvmOptions.setAllJvmArgs(DEFAULT_JVM_ARGS);
+        }
     }
 
     public Map<String, String> getSystemProperties() {
@@ -132,6 +127,7 @@ public class DaemonParameters {
     }
 
     public void setJvmArgs(Iterable<String> jvmArgs) {
+        hasJvmArgs = true;
         jvmOptions.setAllJvmArgs(jvmArgs);
     }
 
