@@ -16,25 +16,17 @@
 
 package org.gradle.model.internal.manage.schema.extract;
 
-import org.gradle.model.internal.manage.schema.ModelSchema;
-import org.gradle.model.internal.manage.schema.cache.ModelSchemaCache;
+import org.gradle.model.internal.manage.schema.ModelSchemaStore;
+import org.gradle.model.internal.manage.schema.ModelValueSchema;
 import org.gradle.model.internal.type.ModelType;
 
-import java.util.Collections;
-
 public class EnumStrategy implements ModelSchemaExtractionStrategy {
 
-    public <T> ModelSchemaExtractionResult<T> extract(ModelSchemaExtractionContext<T> extractionContext, ModelSchemaCache cache) {
+    public <T> void extract(ModelSchemaExtractionContext<T> extractionContext, ModelSchemaStore store) {
         ModelType<T> type = extractionContext.getType();
         if (type.getRawClass().isEnum()) {
-            return new ModelSchemaExtractionResult<T>(ModelSchema.value(type));
-        } else {
-            return null;
+            extractionContext.found(new ModelValueSchema<T>(type));
         }
     }
 
-    public Iterable<String> getSupportedManagedTypes() {
-        return Collections.singleton("enum types");
-    }
-
 }
