@@ -20,16 +20,18 @@ import org.gradle.api.Action;
 import org.gradle.api.NamedDomainObjectFactory;
 import org.gradle.api.Nullable;
 import org.gradle.api.Transformer;
+import org.gradle.api.internal.DefaultPolymorphicNamedEntityInstantiator;
 import org.gradle.api.internal.PolymorphicNamedEntityInstantiator;
-import org.gradle.api.internal.rules.ModelMapCreators;
+import org.gradle.api.internal.rules.DefaultRuleAwarePolymorphicNamedEntityInstantiator;
 import org.gradle.api.internal.rules.RuleAwarePolymorphicNamedEntityInstantiator;
 import org.gradle.internal.*;
 import org.gradle.model.ModelMap;
 import org.gradle.model.RuleSource;
+import org.gradle.model.collection.internal.ChildNodeInitializerStrategyAccessors;
+import org.gradle.model.collection.internal.PolymorphicModelMapProjection;
 import org.gradle.model.internal.core.*;
 import org.gradle.model.internal.core.rule.describe.ModelRuleDescriptor;
 import org.gradle.model.internal.core.rule.describe.SimpleModelRuleDescriptor;
-import org.gradle.model.internal.inspect.DefaultModelCreatorFactory;
 import org.gradle.model.internal.inspect.MethodModelRuleExtractors;
 import org.gradle.model.internal.inspect.ModelRuleExtractor;
 import org.gradle.model.internal.manage.schema.extract.DefaultModelSchemaStore;
@@ -58,7 +60,7 @@ public class ModelRegistryHelper implements ModelRegistry {
     private final ModelRegistry modelRegistry;
 
     public ModelRegistryHelper() {
-        this(new DefaultModelRegistry(new ModelRuleExtractor(MethodModelRuleExtractors.coreExtractors(DefaultModelSchemaStore.getInstance(), new DefaultModelCreatorFactory(DefaultModelSchemaStore.getInstance())))));
+        this(new DefaultModelRegistry(new ModelRuleExtractor(MethodModelRuleExtractors.coreExtractors(DefaultModelSchemaStore.getInstance()))));
     }
 
     public ModelRegistryHelper(ModelRegistryScope modelRegistryScope) {
@@ -118,10 +120,6 @@ public class ModelRegistryHelper implements ModelRegistry {
         return modelRegistry.realizeNode(path);
     }
 
-    public ModelNode realizeNode(String path) {
-        return realizeNode(ModelPath.path(path));
-    }
-
     @Override
     public void remove(ModelPath path) {
         modelRegistry.remove(path);
@@ -163,7 +161,6 @@ public class ModelRegistryHelper implements ModelRegistry {
         return this;
     }
 
-
     @Override
     public ModelRegistryHelper createOrReplace(ModelCreator newCreator) {
         modelRegistry.createOrReplace(newCreator);
@@ -171,11 +168,17 @@ public class ModelRegistryHelper implements ModelRegistry {
     }
 
     @Override
-    public <T> ModelRegistryHelper configure(ModelActionRole role, ModelAction<T> action) {
+    public ModelRegistryHelper configure(ModelActionRole role, ModelAction action) {
         modelRegistry.configure(role, action);
         return this;
     }
 
+    @Override
+    public ModelRegistry configure(ModelActionRole role, ModelAction action, ModelPath scope) {
+        modelRegistry.configure(role, action, scope);
+        return this;
+    }
+
     @Override
     public ModelRegistry apply(Class<? extends RuleSource> rules) {
         return modelRegistry.apply(rules);
@@ -215,7 +218,7 @@ public class ModelRegistryHelper implements ModelRegistry {
     }
 
     public <I> ModelRegistryHelper modelMap(String path, final Class<I> itemType, final Action<? super PolymorphicNamedEntityInstantiator<I>> registrations) {
-        configure(ModelActionRole.Initialize, ModelReference.of(path, ModelMapCreators.instantiatorType(itemType)), new Action<RuleAwarePolymorphicNamedEntityInstantiator<I>>() {
+        configure(ModelActionRole.Initialize, ModelReference.of(path, instantiatorType(itemType)), new Action<RuleAwarePolymorphicNamedEntityInstantiator<I>>() {
             @Override
             public void execute(final RuleAwarePolymorphicNamedEntityInstantiator<I> instantiator) {
                 registrations.execute(new PolymorphicNamedEntityInstantiator<I>() {
@@ -246,9 +249,9 @@ public class ModelRegistryHelper implements ModelRegistry {
     }
 
     public <I> ModelRegistryHelper mutateModelMap(final String path, final Class<I> itemType, final Action<? super ModelMap<I>> action) {
-        return mutate(new Transformer<ModelAction<?>, ModelActionBuilder<Object>>() {
+        return mutate(new Transformer<ModelAction, ModelActionBuilder<Object>>() {
             @Override
-            public ModelAction<?> transform(ModelActionBuilder<Object> builder) {
+            public ModelAction transform(ModelActionBuilder<Object> builder) {
                 return builder.path(path).type(ModelTypes.modelMap(itemType)).action(action);
             }
         });
@@ -266,11 +269,11 @@ public class ModelRegistryHelper implements ModelRegistry {
         return new ModelCreatorBuilder(path);
     }
 
-    public ModelRegistryHelper configure(ModelActionRole role, Transformer<? extends ModelAction<?>, ? super ModelActionBuilder<Object>> def) {
+    public ModelRegistryHelper configure(ModelActionRole role, Transformer<? extends ModelAction, ? super ModelActionBuilder<Object>> def) {
         return configure(role, def.transform(ModelActionBuilder.of()));
     }
 
-    public ModelRegistryHelper mutate(Transformer<? extends ModelAction<?>, ? super ModelActionBuilder<Object>> def) {
+    public ModelRegistryHelper mutate(Transformer<? extends ModelAction, ? super ModelActionBuilder<Object>> def) {
         return configure(Mutate, def);
     }
 
@@ -287,9 +290,9 @@ public class ModelRegistryHelper implements ModelRegistry {
     }
 
     public ModelRegistryHelper mutate(final String path, final Action<? super MutableModelNode> action) {
-        return configure(Mutate, new Transformer<ModelAction<?>, ModelActionBuilder<Object>>() {
+        return configure(Mutate, new Transformer<ModelAction, ModelActionBuilder<Object>>() {
             @Override
-            public ModelAction<?> transform(ModelActionBuilder<Object> objectModelActionBuilder) {
+            public ModelAction transform(ModelActionBuilder<Object> objectModelActionBuilder) {
                 return objectModelActionBuilder.path(path).node(action);
             }
         });
@@ -313,9 +316,9 @@ public class ModelRegistryHelper implements ModelRegistry {
     }
 
     private <T> ModelRegistryHelper configure(ModelActionRole role, final ModelReference<T> reference, final Action<? super T> action) {
-        return configure(role, new Transformer<ModelAction<?>, ModelActionBuilder<Object>>() {
+        return configure(role, new Transformer<ModelAction, ModelActionBuilder<Object>>() {
             @Override
-            public ModelAction<?> transform(ModelActionBuilder<Object> objectModelActionBuilder) {
+            public ModelAction transform(ModelActionBuilder<Object> objectModelActionBuilder) {
                 return objectModelActionBuilder.path(reference.getPath()).type(reference.getType()).action(action);
             }
         });
@@ -333,8 +336,8 @@ public class ModelRegistryHelper implements ModelRegistry {
         return get(path.toString());
     }
 
-    public void realize(String path) {
-        modelRegistry.realize(nonNullValidatedPath(path), ModelType.UNTYPED);
+    public Object realize(String path) {
+        return modelRegistry.realize(nonNullValidatedPath(path), ModelType.UNTYPED);
     }
 
     public <T> T realize(String path, Class<T> type) {
@@ -346,13 +349,13 @@ public class ModelRegistryHelper implements ModelRegistry {
     }
 
     public static <C> ModelCreator creator(String path, final ModelType<C> modelType, String inputPath, final Transformer<? extends C, Object> action) {
-        return ModelCreators.of(ModelPath.path(path), new BiAction<MutableModelNode, List<ModelView<?>>>() {
-            @Override
-            public void execute(MutableModelNode mutableModelNode, List<ModelView<?>> inputs) {
-                mutableModelNode.setPrivateData(modelType, action.transform(inputs.get(0).getInstance()));
-            }
-        }).withProjection(new UnmanagedModelProjection<C>(modelType, true, true))
-            .inputs(refs(ModelReference.of(inputPath)))
+        return ModelCreators.of(ModelPath.path(path), ModelReference.of(inputPath), new BiAction<MutableModelNode, List<ModelView<?>>>() {
+                @Override
+                public void execute(MutableModelNode mutableModelNode, List<ModelView<?>> inputs) {
+                    mutableModelNode.setPrivateData(modelType, action.transform(inputs.get(0).getInstance()));
+                }
+            })
+            .withProjection(new UnmanagedModelProjection<C>(modelType, true, true))
             .descriptor("create " + path)
             .build();
     }
@@ -410,7 +413,7 @@ public class ModelRegistryHelper implements ModelRegistry {
             return copy(type);
         }
 
-        public ModelAction<T> action(final Action<? super T> action) {
+        public ModelAction action(final Action<? super T> action) {
             return build(NO_REFS, new TriAction<MutableModelNode, T, List<ModelView<?>>>() {
                 @Override
                 public void execute(MutableModelNode mutableModelNode, T t, List<ModelView<?>> inputs) {
@@ -419,28 +422,23 @@ public class ModelRegistryHelper implements ModelRegistry {
             });
         }
 
-        public ModelAction<T> node(final Action<? super MutableModelNode> action) {
-            return build(NO_REFS, new TriAction<MutableModelNode, T, List<ModelView<?>>>() {
-                @Override
-                public void execute(MutableModelNode mutableModelNode, T t, List<ModelView<?>> inputs) {
-                    action.execute(mutableModelNode);
-                }
-            });
+        public ModelAction node(final Action<? super MutableModelNode> action) {
+            return toAction(action, path, type, descriptor);
         }
 
-        public <I> ModelAction<T> action(ModelPath modelPath, ModelType<I> inputType, BiAction<? super T, ? super I> action) {
+        public <I> ModelAction action(ModelPath modelPath, ModelType<I> inputType, BiAction<? super T, ? super I> action) {
             return action(modelPath, inputType, inputType.toString(), action);
         }
 
-        public <I> ModelAction<T> action(String modelPath, ModelType<I> inputType, BiAction<? super T, ? super I> action) {
+        public <I> ModelAction action(String modelPath, ModelType<I> inputType, BiAction<? super T, ? super I> action) {
             return action(modelPath, inputType, modelPath, action);
         }
 
-        public <I> ModelAction<T> action(final ModelPath modelPath, final ModelType<I> inputType, String referenceDescription, final BiAction<? super T, ? super I> action) {
+        public <I> ModelAction action(final ModelPath modelPath, final ModelType<I> inputType, String referenceDescription, final BiAction<? super T, ? super I> action) {
             return action(ModelReference.of(modelPath, inputType, referenceDescription), action);
         }
 
-        public <I> ModelAction<T> action(final ModelReference<I> inputReference, final BiAction<? super T, ? super I> action) {
+        public <I> ModelAction action(final ModelReference<I> inputReference, final BiAction<? super T, ? super I> action) {
             return build(refs(inputReference), new TriAction<MutableModelNode, T, List<ModelView<?>>>() {
                 @Override
                 public void execute(MutableModelNode mutableModelNode, T t, List<ModelView<?>> inputs) {
@@ -449,23 +447,23 @@ public class ModelRegistryHelper implements ModelRegistry {
             });
         }
 
-        public <I> ModelAction<T> action(final String modelPath, final ModelType<I> inputType, String referenceDescription, final BiAction<? super T, ? super I> action) {
+        public <I> ModelAction action(final String modelPath, final ModelType<I> inputType, String referenceDescription, final BiAction<? super T, ? super I> action) {
             return action(ModelPath.path(modelPath), inputType, referenceDescription, action);
         }
 
-        public <I> ModelAction<T> action(final ModelType<I> inputType, final BiAction<? super T, ? super I> action) {
+        public <I> ModelAction action(final ModelType<I> inputType, final BiAction<? super T, ? super I> action) {
             return action((ModelPath) null, inputType, action);
         }
 
-        public <I> ModelAction<T> action(final Class<I> inputType, final BiAction<? super T, ? super I> action) {
+        public <I> ModelAction action(final Class<I> inputType, final BiAction<? super T, ? super I> action) {
             return action(ModelType.of(inputType), action);
         }
 
-        private ModelAction<T> build(List<ModelReference<?>> references, TriAction<? super MutableModelNode, ? super T, ? super List<ModelView<?>>> action) {
+        private ModelAction build(List<ModelReference<?>> references, TriAction<? super MutableModelNode, ? super T, ? super List<ModelView<?>>> action) {
             return toAction(references, action, path, type, descriptor);
         }
 
-        private static <T> ModelAction<T> toAction(final List<ModelReference<?>> references, final TriAction<? super MutableModelNode, ? super T, ? super List<ModelView<?>>> action, final ModelPath path, final ModelType<T> type, final ModelRuleDescriptor descriptor) {
+        private static <T> ModelAction toAction(final List<ModelReference<?>> references, final TriAction<? super MutableModelNode, ? super T, ? super List<ModelView<?>>> action, final ModelPath path, final ModelType<T> type, final ModelRuleDescriptor descriptor) {
             return DirectNodeInputUsingModelAction.of(ModelReference.of(path, type), descriptor, references, new TriAction<MutableModelNode, T, List<ModelView<?>>>() {
                 @Override
                 public void execute(MutableModelNode modelNode, T t, List<ModelView<?>> inputs) {
@@ -473,6 +471,10 @@ public class ModelRegistryHelper implements ModelRegistry {
                 }
             });
         }
+
+        private static <T> ModelAction toAction(Action<? super MutableModelNode> action, final ModelPath path, final ModelType<T> type, final ModelRuleDescriptor descriptor) {
+            return DirectNodeNoInputsModelAction.of(ModelReference.of(path, type), descriptor, action);
+        }
     }
 
     public static class ModelCreatorBuilder {
@@ -512,13 +514,13 @@ public class ModelRegistryHelper implements ModelRegistry {
         }
 
         public <C> ModelCreator unmanaged(final ModelType<C> modelType, String inputPath, String inputDescriptor, final Transformer<? extends C, Object> action) {
-            return ModelCreators.of(path, new BiAction<MutableModelNode, List<ModelView<?>>>() {
+            return ModelCreators.of(path, ModelReference.of(inputPath, ModelType.UNTYPED, inputDescriptor), new BiAction<MutableModelNode, List<ModelView<?>>>() {
                 @Override
                 public void execute(MutableModelNode mutableModelNode, List<ModelView<?>> inputs) {
                     mutableModelNode.setPrivateData(modelType, action.transform(inputs.get(0).getInstance()));
                 }
-            }).withProjection(new UnmanagedModelProjection<C>(modelType, true, true))
-                .inputs(ModelReference.of(inputPath, ModelType.UNTYPED, inputDescriptor))
+            })
+                .withProjection(new UnmanagedModelProjection<C>(modelType, true, true))
                 .descriptor(descriptor)
                 .ephemeral(ephemeral)
                 .build();
@@ -529,13 +531,12 @@ public class ModelRegistryHelper implements ModelRegistry {
         }
 
         public <C, I> ModelCreator unmanaged(final ModelType<C> modelType, final ModelType<I> inputModelType, final Transformer<? extends C, ? super I> action) {
-            return ModelCreators.of(path, new BiAction<MutableModelNode, List<ModelView<?>>>() {
+            return ModelCreators.of(path, ModelReference.of(inputModelType), new BiAction<MutableModelNode, List<ModelView<?>>>() {
                 @Override
                 public void execute(MutableModelNode mutableModelNode, List<ModelView<?>> inputs) {
                     mutableModelNode.setPrivateData(modelType, action.transform(ModelViews.assertType(inputs.get(0), inputModelType).getInstance()));
                 }
             }).withProjection(new UnmanagedModelProjection<C>(modelType, true, true))
-                .inputs(ModelReference.of(inputModelType))
                 .ephemeral(ephemeral)
                 .descriptor(descriptor)
                 .build();
@@ -550,12 +551,13 @@ public class ModelRegistryHelper implements ModelRegistry {
         }
 
         private <C> ModelCreator unmanaged(final ModelType<C> modelType, final Factory<? extends C> initializer) {
-            return ModelCreators.of(path, new BiAction<MutableModelNode, List<ModelView<?>>>() {
-                @Override
-                public void execute(MutableModelNode mutableModelNode, List<ModelView<?>> inputs) {
-                    mutableModelNode.setPrivateData(modelType, initializer.create());
-                }
-            }).withProjection(new UnmanagedModelProjection<C>(modelType, true, true))
+            return ModelCreators.of(path, new Action<MutableModelNode>() {
+                    @Override
+                    public void execute(MutableModelNode mutableModelNode) {
+                        mutableModelNode.setPrivateData(modelType, initializer.create());
+                    }
+                })
+                .withProjection(new UnmanagedModelProjection<C>(modelType, true, true))
                 .descriptor(descriptor)
                 .ephemeral(ephemeral)
                 .build();
@@ -566,12 +568,13 @@ public class ModelRegistryHelper implements ModelRegistry {
         }
 
         public <C> ModelCreator unmanagedNode(ModelType<C> modelType, final Action<? super MutableModelNode> action) {
-            return ModelCreators.of(path, new BiAction<MutableModelNode, List<ModelView<?>>>() {
-                @Override
-                public void execute(MutableModelNode mutableModelNode, List<ModelView<?>> inputs) {
-                    action.execute(mutableModelNode);
-                }
-            }).withProjection(new UnmanagedModelProjection<C>(modelType, true, true))
+            return ModelCreators.of(path, new Action<MutableModelNode>() {
+                    @Override
+                    public void execute(MutableModelNode mutableModelNode) {
+                        action.execute(mutableModelNode);
+                    }
+                })
+                .withProjection(new UnmanagedModelProjection<C>(modelType, true, true))
                 .descriptor(descriptor)
                 .ephemeral(ephemeral)
                 .build();
@@ -591,12 +594,35 @@ public class ModelRegistryHelper implements ModelRegistry {
             });
         }
 
-        public <I> ModelCreator modelMap(Class<I> itemType) {
-            return ModelMapCreators.of(path, itemType)
+        public <I> ModelCreator modelMap(final Class<I> itemType) {
+            final ModelType<RuleAwarePolymorphicNamedEntityInstantiator<I>> instantiatorType = instantiatorType(itemType);
+
+            ModelType<I> modelType = ModelType.of(itemType);
+            return ModelCreators.of(
+                ModelReference.of(path, instantiatorType),
+                new Factory<RuleAwarePolymorphicNamedEntityInstantiator<I>>() {
+                    @Override
+                    public RuleAwarePolymorphicNamedEntityInstantiator<I> create() {
+                        return new DefaultRuleAwarePolymorphicNamedEntityInstantiator<I>(
+                            new DefaultPolymorphicNamedEntityInstantiator<I>(itemType, "this collection")
+                        );
+                    }
+                }
+            )
+                .withProjection(PolymorphicModelMapProjection.of(
+                    modelType,
+                    ChildNodeInitializerStrategyAccessors.constant(NodeBackedModelMap.createUsingParentNode(modelType)))
+                )
+                .withProjection(UnmanagedModelProjection.of(instantiatorType))
                 .descriptor(descriptor)
                 .ephemeral(ephemeral)
                 .build();
         }
     }
 
+    public static <T> ModelType<RuleAwarePolymorphicNamedEntityInstantiator<T>> instantiatorType(Class<T> typeClass) {
+        return new ModelType.Builder<RuleAwarePolymorphicNamedEntityInstantiator<T>>() {
+        }.where(new ModelType.Parameter<T>() {
+        }, ModelType.of(typeClass)).build();
+    }
 }
